function out = branchpoint1
%
% Branch point curve definition file for a problem in odefile
% 

    out{1}  = @curve_func;
    out{2}  = @defaultprocessor;
    out{3}  = @options;
    out{4}  = @jacobian;
    out{5}  = @hessians;
    out{6}  = @testf;
    out{7}  = @userf;
    out{8}  = @process;
    out{9}  = @singmat;
    out{10} = @locate;
    out{11} = @init;
    out{12} = @done;
    out{13} = @adapt;
return

%----------------------------------------------------
function func = curve_func(arg)
global  bpds
  [x,p] = rearr(arg); p = num2cell(p);
  Bord=[cjac(bpds.func,bpds.Jacobian,x,p,bpds.ActiveParams) cjacbr(bpds.func,bpds.JacobianP,x,p,bpds.ActiveParams,bpds.BranchParam) bpds.borders.w;bpds.borders.v' zeros(2,1)];
  bunit=[zeros(bpds.nphase,2);eye(2)];
  vext=Bord\bunit;
  func = [feval(bpds.func, 0, x, p{:}); vext(end,1);vext(end,2)];

%------------------------------------------------------
function jac = jacobian(varargin)
global  bpds
  nap = length(bpds.ActiveParams);
  xo = varargin{1}; [x,p] = rearr(xo); p = num2cell(p);
  jac = cjac(bpds.func,bpds.Jacobian,x,p,bpds.ActiveParams);
  Bord = [jac cjacbr(bpds.func,bpds.JacobianP,x,p,bpds.ActiveParams,bpds.BranchParam) bpds.borders.w;bpds.borders.v' zeros(2,1)];
  bunit = [zeros(bpds.nphase,2);eye(2)];
  vext = Bord\bunit;
  bunit=[zeros(bpds.nphase+1,1);1];
  wext = Bord'\bunit;
  jac = [jac cjacp(bpds.func,bpds.JacobianP,x,p,bpds.ActiveParams)];
  hess  = chess(bpds.func,bpds.Jacobian,bpds.Hessians,x,p,bpds.ActiveParams);
  hessp = chessp(bpds.func,bpds.Jacobian,bpds.HessiansP,x,p,bpds.ActiveParams);
  hessbr= chessbr(bpds.func,bpds.Jacobian,bpds.HessiansP,x,p,bpds.ActiveParams,bpds.BranchParam);
  for i=1:bpds.nphase
     jac(bpds.nphase+1,i)=-wext(1:bpds.nphase)'*hess(:,:,i)*vext(1:bpds.nphase,1)-wext(1:bpds.nphase)'*hessbr(:,i)*vext(end-1,1);
     jac(bpds.nphase+2,i)=-wext(1:bpds.nphase)'*hess(:,:,i)*vext(1:bpds.nphase,2)-wext(1:bpds.nphase)'*hessbr(:,i)*vext(end-1,2);
  end
  hesspp = chesspbr(bpds.func,bpds.JacobianP,bpds.HessiansP,x,p,bpds.ActiveParams,bpds.BranchParam);
  for i=1:nap
      jac(bpds.nphase+1,bpds.nphase+i)=-wext(1:bpds.nphase)'*hessp(:,:,i)*vext(1:bpds.nphase,1)-wext(1:bpds.nphase)'*hesspp(:,:,i)*vext(end-1,1);
      jac(bpds.nphase+2,bpds.nphase+i)=-wext(1:bpds.nphase)'*hessp(:,:,i)*vext(1:bpds.nphase,2)-wext(1:bpds.nphase)'*hesspp(:,:,i)*vext(end-1,2);     
  end
  
%-------------------------------------------------------
function hessians()
  
%-------------------------------------------------------  
function varargout = defaultprocessor(varargin)
global cds bpds
  if nargin > 2
    s = varargin{3};
    varargout{3} = s;
  end

 % compute eigenvalues?
  if (cds.options.Eigenvalues==1)
      xo = varargin{1}; [x,p] = rearr(xo); p = num2cell(p);
      d = eig(cjac(bpds.func,bpds.Jacobian,x,p,bpds.ActiveParams));
      [Y,I] = sort(real(d));
      varargout{2} = d(I);
  else
      varargout{2} = nan;
  end  

  % all done succesfully
  varargout{1} = 0;

%----------------------------------------------------------
function option = options
global bpds
  option = contset;
  
  bp=1;
  switch bpds.nphase
      case 1 % in R1 non ci possono essere BT e ZH
          option=contset(option,'IgnoreSingularity',[bp bp]);
      case 2 % in R2 non ci pu essere la ZH
          option=contset(option,'IgnoreSingularity',[bp+1]);
  end

  % Check for symbolic derivatives in odefile
  symord = 0; 
  if ~isempty(bpds.Jacobian), symord = 1; end
  if ~isempty(bpds.Hessians), symord = 2; end
  if ~isempty(bpds.Der3), symord = 3; end
  if ~isempty(bpds.Der4), symord = 4; end
  if ~isempty(bpds.Der5), symord = 5; end
  option = contset(option, 'SymDerivative', symord);
  option = contset(option, 'Workspace', 1);
  option = contset(option, 'Locators', [0 0 0]);
  
  symordp = 0;
  if ~isempty(bpds.HessiansP)
      symordp = 2;
  elseif ~isempty(bpds.JacobianP)  
      symordp = 1; 
  end
  option = contset(option,'SymDerivativeP',symordp);



  
%-----------------------------------------------------
function [out,failed] = testf(id, x, v)
global bpds
  [x0,p] = rearr(x); p = num2cell(p);
  jac=cjac(bpds.func,bpds.Jacobian,x0,p,bpds.ActiveParams);
  vbord=bpds.borders.v(1:bpds.nphase,1)*bpds.borders.v(bpds.nphase+1,2)...
       -bpds.borders.v(1:bpds.nphase,2)*bpds.borders.v(bpds.nphase+1,1);
  Bord=[jac bpds.borders.w;vbord' 0];
  bunit=[zeros(bpds.nphase,1);1];
  vext=Bord\bunit;
  vext=vext(1:bpds.nphase);
  wext=Bord'\bunit;
  wext=wext(1:bpds.nphase);
  failed = [];
  
if bpds.nphase > 1
if ismember(1,id)% case 1 % BT
    out(1) = wext'*vext;
end
end

if bpds.nphase > 2
if ismember(2,id)% case 2 % ZH
    A = jac;
    A1=sparse(bpds.BiAlt_M1_I,bpds.BiAlt_M1_J,A(bpds.BiAlt_M1_V));
    A2=sparse(bpds.BiAlt_M2_I,bpds.BiAlt_M2_J,A(bpds.BiAlt_M2_V));
    A3=sparse(bpds.BiAlt_M3_I,bpds.BiAlt_M3_J,A(bpds.BiAlt_M3_V));
    A = A1-A2+A3;
    bigmat = [A bpds.bigW; bpds.bigV' bpds.bigD];
    
    Xg = bigmat \ [zeros(bpds.nphase*(bpds.nphase-1)/2,1); 1];
    out(2) = Xg(end);
end
end

if ismember(3,id)% case 3 % CP
    out(3) = nf_LP(bpds.func,bpds.Jacobian,bpds.Hessians,x0,p,vext/norm(vext),wext/(wext'*vext),bpds.nphase);
end


%-----------------------------------------------------
function [out, failed] = userf(userinf, id, x, v)
global  bpds
dim =size(id,2);
failed = [];
out = zeros(1,dim);
for i=1:dim
  lastwarn('');
  [x0,p] = rearr(x); p = num2cell(p);
  if (userinf(i).state==1)
      out(i)=feval(bpds.user{id(i)},0,x0,p{1:end-1});
  end
  if ~isempty(lastwarn)
    failed = [failed i];
  end
end
%---------------------------------------------------------
function [failed,s] = process(id, x, v, s)
global cds bpds
  ndim = cds.ndim;
  [x0,p] = rearr(x); p = num2cell(p);
% WM: Removed SL array
fprintf('label = %s, x = ', s.label); printv(x);

switch id
  case 1 % BT
    % s.data.c = nf_BT(bpds.func,bpds.Jacobian,bpds.Hessians,x0,p,bpds.nphase);
    % fprintf('(a,b)=(%d, %d)\n',s.data.c);
    s.msg  = sprintf('Bogdanov-Takens point');
  case 2 % ZH
    % s.data.c = nf_ZH(bpds.func,bpds.Jacobian,bpds.Hessians,bpds.Der3,x0,p,bpds.nphase);
    if ~all(real(s.data.c) == 0)
    %    fprintf('(s,theta,E0)=(%d, %d, %d)\n',real(s.data.c));
        s.msg = sprintf('Zero-Hopf point');
    else
        fprintf('Zero-Neutral Saddle\n');
        s.msg  = sprintf('Zero-Hopf point: neutral saddle');
    end
  case 3 % CP
    global lpds;
    lpds=bpds;
    s.data.c = nf_CP(bpds.func,bpds.Jacobian,bpds.Hessians,bpds.Der3,x0,p,bpds.nphase);
    fprintf('c=%d\n',s.data.c);
    s.msg  = sprintf('Cusp point');
%   otherwise
%     s.msg = sprintf('Branch point (parameter %d)',bpds.BranchParams(id-3));
end

% Compute eigenvalues for every singularity
[x0,p] = rearr(x); p = num2cell(p); 
J=cjac(bpds.func,bpds.Jacobian,x0,p,bpds.ActiveParams); 
if ~issparse(J)
  [v,d]=eig(J);
else
  opt.disp=0;
  % WM: fixed a bug (incompatability between MatLab 6.0 and 5.5?)
  [v,d]=eigs(J,min(6,ndim-2),'lm',opt);
end

s.data.evec = v;
s.data.eval = diag(d)';

failed = 0;
%---------------------------------------------------------
function [S,L] = singmat(varargin)
global bpds 
% 0: testfunction must vanish
% 1: testfunction must not vanish
% everything else: ignore this testfunction

  S = [  0 0 8 
         1 0 8 
         8 8 0 ];
     if (bpds.nphase==2)
         S(1,2)=8;
     end
  L = [ 'BT '; 'ZH '; 'CP '];
  
%---------------------------------------------------------
function [x,v] = locate(id, varargin)
error('No locator defined for singularity %d', id);
%---------------------------------------------------------
function varargout = init(varargin)
  x = varargin{1};
  v = varargin{2};
  WorkspaceInit(x,v);

  % all done succesfully
  varargout{1} = 0;
%---------------------------------------------------------
function done
  WorkspaceDone;

% -------------------------------------------------------
function [res,x,v] = adapt(x,v)
global bpds
[x1,p] =rearr(x); p = num2cell(p);
jac=cjac(bpds.func,bpds.Jacobian,x1,p,bpds.ActiveParams);
Bord=[jac cjacbr(bpds.func,bpds.JacobianP,x1,p,bpds.ActiveParams,bpds.BranchParam) bpds.borders.w;bpds.borders.v' zeros(2,1)];
bunit=[zeros(bpds.nphase,2);eye(2)];
vext=Bord\bunit;
bunit=[zeros(bpds.nphase+1,1);1];
wext = Bord'\bunit;
bpds.borders.w=wext(1:bpds.nphase);
bpds.borders.v=vext(1:bpds.nphase+1,:)/norm(vext(1:bpds.nphase+1,:));
res = []; % no re-evaluations needed
bpds.joinvector=wext(1:bpds.nphase);

if bpds.nphase > 2
    A1 = sparse(bpds.BiAlt_M1_I,bpds.BiAlt_M1_J,jac(bpds.BiAlt_M1_V));
    A2 = sparse(bpds.BiAlt_M2_I,bpds.BiAlt_M2_J,jac(bpds.BiAlt_M2_V));
    A3 = sparse(bpds.BiAlt_M3_I,bpds.BiAlt_M3_J,jac(bpds.BiAlt_M3_V));
    A = A1-A2+A3;
    [Q,R,E] = qr(full(A));
    if bpds.bigW' * Q(:,end) < 0
        bpds.bigW = -Q(:,end);
    else
        bpds.bigW = Q(:,end);
    end
    if bpds.bigV' * E(:,end) < 0
        bpds.bigV = -E(:,end);
    else
        bpds.bigV = E(:,end);
    end
end

res = 1; 
  
%----------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ---------------------------------------------------------------

function [x,p] = rearr(x0)
%
% [x,p] = rearr(x0)
%
% Rearranges x0 into coordinates (x) and parameters (p)
global bpds
p = bpds.P0;
p(bpds.ActiveParams) = x0((bpds.nphase+1):end);
x = x0(1:bpds.nphase);

% ---------------------------------------------------------

function WorkspaceInit(x,v)
global bpds 

% calculate some matrices to efficiently compute bialternate products
% (without loops)
n = bpds.nphase;
a = reshape(1:(n^2),n,n);
[bia,bin,bip] = bialt(a);
if any(any(bip))
    [bpds.BiAlt_M1_I,bpds.BiAlt_M1_J,bpds.BiAlt_M1_V] = find(bip);
else
    bpds.BiAlt_M1_I=1;bpds.BiAlt_M1_J=1;bpds.BiAlt_M1_V=n^2+1;
end    
if any(any(bin))
    [bpds.BiAlt_M2_I,bpds.BiAlt_M2_J,bpds.BiAlt_M2_V] = find(bin);
else
     bpds.BiAlt_M2_I=1;bpds.BiAlt_M2_J=1;bpds.BiAlt_M2_V=n^2+1;
end
if any(any(bia))
    [bpds.BiAlt_M3_I,bpds.BiAlt_M3_J,bpds.BiAlt_M3_V] = find(bia);
else
    bpds.BiAlt_M3_I=1;bpds.BiAlt_M3_J=1;bpds.BiAlt_M3_V=n^2+1;
end


% ------------------------------------------------------

function WorkspaceDone


%SD:continues branchpoint of odefile
