function out = prey_predator2020
out{1} = @init;
out{2} = @fun_eval;
out{3} = @jacobian;
out{4} = @jacobianp;
out{5} = @hessians;
out{6} = @hessiansp;
out{7} = @der3;
out{8} = [];
out{9} = [];

% --------------------------------------------------------------------------
function dydt = fun_eval(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
dydt=[rr*kmrgd(1)*(1-kmrgd(1)/KK)-a1*kmrgd(1)*kmrgd(2)/(b1+kmrgd(1));
ee*a1*kmrgd(1)*kmrgd(2)/(b1+kmrgd(1))-mm*kmrgd(2)-a2*kmrgd(2)*x3/(b2+kmrgd(2));];

% --------------------------------------------------------------------------
function [tspan,y0,options] = init
handles = feval(prey_predator2020);
y0=[0,0];
options = odeset('Jacobian',handles(3),'JacobianP',handles(4),'Hessians',handles(5),'HessiansP',handles(6));
tspan = [0 10];

% --------------------------------------------------------------------------
function jac = jacobian(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
jac=[ (a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 - (a1*kmrgd(2))/(b1 + kmrgd(1)) - (rr*kmrgd(1))/KK - rr*(kmrgd(1)/KK - 1) , -(a1*kmrgd(1))/(b1 + kmrgd(1)) ; (a1*ee*kmrgd(2))/(b1 + kmrgd(1)) - (a1*ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 , (a1*ee*kmrgd(1))/(b1 + kmrgd(1)) - (a2*x3)/(b2 + kmrgd(2)) - mm + (a2*kmrgd(2)*x3)/(b2 + kmrgd(2))^2 ];
% --------------------------------------------------------------------------
function jacp = jacobianp(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
jacp=[ 0 , -kmrgd(1)*(kmrgd(1)/KK - 1) , (rr*kmrgd(1)^2)/KK^2 , -(kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1)) , (a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 , 0 , 0 , 0 , 0 ; -(a2*kmrgd(2))/(b2 + kmrgd(2)) , 0 , 0 , (ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1)) , -(a1*ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 , (a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1)) , -kmrgd(2) , -(kmrgd(2)*x3)/(b2 + kmrgd(2)) , (a2*kmrgd(2)*x3)/(b2 + kmrgd(2))^2 ];
% --------------------------------------------------------------------------
function hess = hessians(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
hess1=[ (2*a1*kmrgd(2))/(b1 + kmrgd(1))^2 - (2*rr)/KK - (2*a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^3 , (a1*kmrgd(1))/(b1 + kmrgd(1))^2 - a1/(b1 + kmrgd(1)) ; (2*a1*ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^3 - (2*a1*ee*kmrgd(2))/(b1 + kmrgd(1))^2 , (a1*ee)/(b1 + kmrgd(1)) - (a1*ee*kmrgd(1))/(b1 + kmrgd(1))^2 ];
hess2=[ (a1*kmrgd(1))/(b1 + kmrgd(1))^2 - a1/(b1 + kmrgd(1)) , 0 ; (a1*ee)/(b1 + kmrgd(1)) - (a1*ee*kmrgd(1))/(b1 + kmrgd(1))^2 , (2*a2*x3)/(b2 + kmrgd(2))^2 - (2*a2*kmrgd(2)*x3)/(b2 + kmrgd(2))^3 ];
hess(:,:,1) =hess1;
hess(:,:,2) =hess2;
% --------------------------------------------------------------------------
function hessp = hessiansp(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
hessp1=[ 0 , 0 ; 0 , (a2*kmrgd(2))/(b2 + kmrgd(2))^2 - a2/(b2 + kmrgd(2)) ];
hessp2=[ 1 - (2*kmrgd(1))/KK , 0 ; 0 , 0 ];
hessp3=[ (2*rr*kmrgd(1))/KK^2 , 0 ; 0 , 0 ];
hessp4=[ (kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 - kmrgd(2)/(b1 + kmrgd(1)) , -kmrgd(1)/(b1 + kmrgd(1)) ; (ee*kmrgd(2))/(b1 + kmrgd(1)) - (ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 , (ee*kmrgd(1))/(b1 + kmrgd(1)) ];
hessp5=[ (a1*kmrgd(2))/(b1 + kmrgd(1))^2 - (2*a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^3 , (a1*kmrgd(1))/(b1 + kmrgd(1))^2 ; (2*a1*ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^3 - (a1*ee*kmrgd(2))/(b1 + kmrgd(1))^2 , -(a1*ee*kmrgd(1))/(b1 + kmrgd(1))^2 ];
hessp6=[ 0 , 0 ; (a1*kmrgd(2))/(b1 + kmrgd(1)) - (a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^2 , (a1*kmrgd(1))/(b1 + kmrgd(1)) ];
hessp7=[ 0 , 0 ; 0 , -1 ];
hessp8=[ 0 , 0 ; 0 , (kmrgd(2)*x3)/(b2 + kmrgd(2))^2 - x3/(b2 + kmrgd(2)) ];
hessp9=[ 0 , 0 ; 0 , (a2*x3)/(b2 + kmrgd(2))^2 - (2*a2*kmrgd(2)*x3)/(b2 + kmrgd(2))^3 ];
hessp(:,:,1) =hessp1;
hessp(:,:,2) =hessp2;
hessp(:,:,3) =hessp3;
hessp(:,:,4) =hessp4;
hessp(:,:,5) =hessp5;
hessp(:,:,6) =hessp6;
hessp(:,:,7) =hessp7;
hessp(:,:,8) =hessp8;
hessp(:,:,9) =hessp9;
%---------------------------------------------------------------------------
function tens3  = der3(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
tens31=[ (6*a1*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^4 - (6*a1*kmrgd(2))/(b1 + kmrgd(1))^3 , (2*a1)/(b1 + kmrgd(1))^2 - (2*a1*kmrgd(1))/(b1 + kmrgd(1))^3 ; (6*a1*ee*kmrgd(2))/(b1 + kmrgd(1))^3 - (6*a1*ee*kmrgd(1)*kmrgd(2))/(b1 + kmrgd(1))^4 , (2*a1*ee*kmrgd(1))/(b1 + kmrgd(1))^3 - (2*a1*ee)/(b1 + kmrgd(1))^2 ];
tens32=[ (2*a1)/(b1 + kmrgd(1))^2 - (2*a1*kmrgd(1))/(b1 + kmrgd(1))^3 , 0 ; (2*a1*ee*kmrgd(1))/(b1 + kmrgd(1))^3 - (2*a1*ee)/(b1 + kmrgd(1))^2 , 0 ];
tens33=[ (2*a1)/(b1 + kmrgd(1))^2 - (2*a1*kmrgd(1))/(b1 + kmrgd(1))^3 , 0 ; (2*a1*ee*kmrgd(1))/(b1 + kmrgd(1))^3 - (2*a1*ee)/(b1 + kmrgd(1))^2 , 0 ];
tens34=[ 0 , 0 ; 0 , (6*a2*kmrgd(2)*x3)/(b2 + kmrgd(2))^4 - (6*a2*x3)/(b2 + kmrgd(2))^3 ];
tens3(:,:,1,1) =tens31;
tens3(:,:,1,2) =tens32;
tens3(:,:,2,1) =tens33;
tens3(:,:,2,2) =tens34;
%---------------------------------------------------------------------------
function tens4  = der4(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
%---------------------------------------------------------------------------
function tens5  = der5(t,kmrgd,x3,rr,KK,a1,b1,ee,mm,a2,b2)
